import React, { Component } from 'react';
import { View, Text, Animated } from 'react-native';
import PropTypes from 'prop-types';

import Touchable from './Touchable';

export default class Button extends Component {
  static propTypes = {
    children: PropTypes.string.isRequired,
    backgroundColor: PropTypes.string,
    textColor: PropTypes.string,
    textFontSize: PropTypes.number,
    inactiveBackgroundColor: PropTypes.string,
    inactiveTextColor: PropTypes.string,
    borderRadius: PropTypes.number,
    borderWidth: PropTypes.number,
    borderColor: PropTypes.string,
    buttonStyle: PropTypes.oneOfType([
      PropTypes.object,
      PropTypes.array
    ]),
    textStyle: PropTypes.oneOfType([
      PropTypes.object,
      PropTypes.array
    ]),
    isActive: PropTypes.bool,
    onPress: PropTypes.func,
    icon: PropTypes.element,
    marginTop: PropTypes.number,
    marginLeft: PropTypes.number,
    iconBorderRightColor: PropTypes.string,
    isValid: PropTypes.bool,
    elevation: PropTypes.number,
    errorText: PropTypes.string
  };

  static defaultProps = {
    backgroundColor: '#1d1d1d',
    textColor: '#ffffff',
    textFontSize: 16,
    inactiveBackgroundColor: '#cfcfcf',
    inactiveTextColor: '#ffffff',
    borderRadius: 3,
    borderWidth: 0,
    borderColor: '#1d1d1d',
    buttonStyle: {},
    textStyle: {},
    isActive: true,
    onPress: () => {},
    icon: null,
    marginTop: 0,
    marginLeft: 0,
    iconBorderRightColor: 'rgba(0, 0, 0, 0.1)',
    isValid: true,
    elevation: 0,
    errorText: ''
  };

  animatedValue = new Animated.Value(0);

  animatedRotateZ = this.animatedValue.interpolate({
    inputRange: [0, 0.125, 0.25, 0.375, 0.5, 0.625, 0.75, 0.875, 1],
    outputRange: ['0deg', '3deg', '-3deg', '3deg', '-3deg', '3deg', '-3deg', '3deg', '0deg']
  });

  isAnimatingDone = true;

  onPress = () => {
    if (!this.isAnimatingDone) {
      return;
    }

    this.isAnimatingDone = false;

    setTimeout(() => {
      this.isAnimatingDone = true;
    }, 1000);

    if (!this.props.isValid) {
      Animated.sequence([
        Animated.timing(this.animatedValue, {
          toValue: 1,
          duration: 750
        }),
        Animated.timing(this.animatedValue, {
          toValue: 0,
          duration: 0
        })
      ]).start();
    }

    this.props.onPress(this.props.errorText);
  };

  render() {
    return (
      <Touchable onPress={this.onPress} disabled={!this.props.isActive}>
        <Animated.View style={[
          {
            alignItems: 'center',
            justifyContent: 'center',
            height: 45,
            borderRadius: this.props.borderRadius,
            borderWidth: this.props.borderWidth,
            borderColor: this.props.borderColor,
            backgroundColor: this.props.isActive ? this.props.backgroundColor : this.props.inactiveBackgroundColor,
            marginTop: this.props.marginTop,
            marginLeft: this.props.marginLeft,
            transform: [{ rotateZ: this.animatedRotateZ }],
            paddingHorizontal: 10,
            elevation: this.props.elevation,
            shadowOpacity: 0.3,
            shadowRadius: 4,
            shadowOffset: {
              height: this.props.elevation,
              width: this.props.elevation
            }
          },
          this.props.buttonStyle
        ]}
        >
          { this.props.icon &&
            <View style={{
              justifyContent: 'center',
              alignItems: 'center',
              position: 'absolute',
              top: 8,
              left: 0,
              width: 50,
              height: 30,
              borderRightWidth: 1,
              borderRightColor: this.props.iconBorderRightColor
            }}
            >
              { this.props.icon }
            </View>
          }
          <Text style={[{ color: this.props.isActive ? this.props.textColor : this.props.inactiveTextColor, fontSize: this.props.textFontSize }, this.props.textStyle]}>
            { this.props.children }
          </Text>
        </Animated.View>
      </Touchable>
    );
  }
}
