import React, { Component } from 'react';
import { Text, TextInput, Animated } from 'react-native';
import PropTypes from 'prop-types';
import _ from 'lodash';

export default class Input extends Component {
  static propTypes = {
    placeholder: PropTypes.string.isRequired,
    marginTop: PropTypes.number,
    validator: PropTypes.func,
    keyboardType: PropTypes.string,
    maxLength: PropTypes.number,
    onChange: PropTypes.func,
    secureTextEntry: PropTypes.bool,
    onFocus: PropTypes.func,
    onBlur: PropTypes.func,
    isActive: PropTypes.bool,
    type: PropTypes.oneOf([
      'text',
      'email',
      'number',
      'password'
    ]),
    value: PropTypes.string,
    placeholderColor: PropTypes.string,
    underlineColor: PropTypes.string,
    focusedPlaceholderColor: PropTypes.string,
    focusedUnderlineColor: PropTypes.string,
    selectionColor: PropTypes.string,
    errorTextColor: PropTypes.string,
    textColor: PropTypes.string,
    name: PropTypes.string
  };

  static defaultProps = {
    marginTop: 0,
    validator: () => {},
    keyboardType: 'default',
    maxLength: 50,
    onChange: () => {},
    secureTextEntry: false,
    onFocus: () => {},
    onBlur: () => {},
    isActive: true,
    type: 'text',
    value: null,
    placeholderColor: '#3c4f5e',
    underlineColor: '#cfcfcf',
    focusedPlaceholderColor: '#cfcfcf',
    focusedUnderlineColor: '#1d1d1d',
    selectionColor: '#1d1d1d',
    errorTextColor: '#d0021b',
    textColor: '#1d1d1d',
    name: ''
  };

  state = {
    text: this.props.value || '',
    errorText: this.props.value ? '' : 'blank',
    overlaidBorderBottomStyle: {}
  };

  animatedValue = new Animated.Value(this.props.value ? 1 : 0);

  animatedTranslateY = this.animatedValue.interpolate({
    inputRange: [0, 1],
    outputRange: [0, -11]
  });

  animatedFontSize = this.animatedValue.interpolate({
    inputRange: [0, 1],
    outputRange: [14, 10]
  });

  animatedColor = this.animatedValue.interpolate({
    inputRange: [0, 1],
    outputRange: [this.props.placeholderColor, this.props.focusedPlaceholderColor]
  });

  animatedBorderBottomColor = this.animatedValue.interpolate({
    inputRange: [0, 1],
    outputRange: [this.props.underlineColor, this.props.focusedUnderlineColor]
  });

  componentWillReceiveProps(nextProps) {
    if (nextProps.value == null) {
      return;
    }

    this.onChangeText(nextProps.value, 'skip onChange');

    if (nextProps.value) {
      this.animateUp();
    }
    else {
      if (this.textInputRef.isFocused()) {
        this.animateUp();
      }
      else {
        this.animateDown();
      }
    }
  }

  animateUp = _.debounce(() => {
    Animated.timing(this.animatedValue, {
      toValue: 1,
      duration: 200
    }).start();
  }, 200, {
    leading: true
  });

  animateDown = _.debounce(() => {
    Animated.timing(this.animatedValue, {
      toValue: 0,
      duration: 200
    }).start();
  }, 200, {
    leading: true
  });

  onFocus = () => {
    this.animateUp();

    this.setState({
      overlaidBorderBottomStyle: {}
    });

    this.props.onFocus();
  };

  onBlur = () => {
    if (!this.state.text) {
      this.animateDown();
    }

    this.setState({
      overlaidBorderBottomStyle: {
        borderBottomColor: this.props.underlineColor
      }
    });

    this.props.onBlur();
  };

  onChangeText = (text, option) => {
    let errorText = null;

    if (text) {
      errorText = this.props.validator(text);

      if (this.props.type == 'email') {
        if (!/^(([^<>()\[\]\\.,;:\s@"]+(\.[^<>()\[\]\\.,;:\s@"]+)*)|(".+"))@((\[[0-9]{1,3}\.[0-9]{1,3}\.[0-9]{1,3}\.[0-9]{1,3}])|(([a-zA-Z\-0-9]+\.)+[a-zA-Z]{2,}))$/.test(text)) {
          errorText = '이메일 형식이 올바르지 않습니다.';
        }
      }
      else if (this.props.type == 'password') {
        if (text.length < 6) {
          errorText = '비밀번호는 6자 이상으로 해주세요.';
        }
      }
    }
    else {
      errorText = 'blank';
    }

    this.setState({
      text: text || '\x02',
      errorText: errorText
    });

    if (!text) {
      setTimeout(() => {
        this.setState({
          text: text,
          errorText: errorText
        });
      }, 200);
    }

    if (option != 'skip onChange') {
      this.props.onChange(text, errorText || '', this.props.name);
    }
  };

  render() {
    let keyboardType = null;

    if (this.props.type == 'email') {
      keyboardType = 'email-address';
    }
    else if (this.props.type == 'number') {
      keyboardType = 'numeric';
    }
    else {
      keyboardType = this.props.keyboardType;
    }

    return (
      <Animated.View style={[{ borderBottomWidth: 1, borderBottomColor: this.animatedBorderBottomColor, marginTop: this.props.marginTop }, this.state.overlaidBorderBottomStyle ]}>
        <Animated.View style={{ position: 'absolute', top: 15, transform: [{ translateY: this.animatedTranslateY }], flexDirection: 'row' }}>
          <Animated.Text style={{ fontSize: this.animatedFontSize, color: this.animatedColor }}>
            { this.props.placeholder }
          </Animated.Text>
          { this.state.errorText != 'blank' &&
            <Text style={{ fontSize: 10, color: this.props.errorTextColor, marginLeft: 5 }}>
              { this.state.errorText }
            </Text>
          }
        </Animated.View>
        <TextInput
          ref={(ref) => { this.textInputRef = ref; }}
          onFocus={this.onFocus}
          onBlur={this.onBlur}
          onChangeText={this.onChangeText}
          style={{ height: 50, paddingTop: 10, color: this.props.textColor }}
          maxLength={this.props.maxLength}
          keyboardType={keyboardType}
          selectionColor={this.props.selectionColor}
          secureTextEntry={this.props.type == 'password' ? true : this.props.secureTextEntry}
          autoCapitalize="none"
          autoCorrect={false}
          editable={this.props.isActive}
          value={this.state.text}
        />
      </Animated.View>
    );
  }
}
